# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_metadata_model_classification_metadata_model import IMetadataModelClassificationMetadataModel
from rapidata.api_client.models.i_metadata_model_count_metadata_model import IMetadataModelCountMetadataModel
from rapidata.api_client.models.i_metadata_model_file_type_metadata_model import IMetadataModelFileTypeMetadataModel
from rapidata.api_client.models.i_metadata_model_image_dimension_metadata_model import IMetadataModelImageDimensionMetadataModel
from rapidata.api_client.models.i_metadata_model_location_metadata_model import IMetadataModelLocationMetadataModel
from rapidata.api_client.models.i_metadata_model_original_filename_metadata_model import IMetadataModelOriginalFilenameMetadataModel
from rapidata.api_client.models.i_metadata_model_source_url_metadata_model import IMetadataModelSourceUrlMetadataModel
from rapidata.api_client.models.i_metadata_model_streams_metadata_model import IMetadataModelStreamsMetadataModel
from rapidata.api_client.models.i_metadata_model_text_metadata_model import IMetadataModelTextMetadataModel
from rapidata.api_client.models.i_metadata_model_video_duration_metadata_model import IMetadataModelVideoDurationMetadataModel
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

IMETADATAMODEL_ONE_OF_SCHEMAS = ["IMetadataModelClassificationMetadataModel", "IMetadataModelCountMetadataModel", "IMetadataModelFileTypeMetadataModel", "IMetadataModelImageDimensionMetadataModel", "IMetadataModelLocationMetadataModel", "IMetadataModelOriginalFilenameMetadataModel", "IMetadataModelSourceUrlMetadataModel", "IMetadataModelStreamsMetadataModel", "IMetadataModelTextMetadataModel", "IMetadataModelVideoDurationMetadataModel"]

class IMetadataModel(BaseModel):
    """
    IMetadataModel
    """
    # data type: IMetadataModelClassificationMetadataModel
    oneof_schema_1_validator: Optional[IMetadataModelClassificationMetadataModel] = None
    # data type: IMetadataModelCountMetadataModel
    oneof_schema_2_validator: Optional[IMetadataModelCountMetadataModel] = None
    # data type: IMetadataModelFileTypeMetadataModel
    oneof_schema_3_validator: Optional[IMetadataModelFileTypeMetadataModel] = None
    # data type: IMetadataModelImageDimensionMetadataModel
    oneof_schema_4_validator: Optional[IMetadataModelImageDimensionMetadataModel] = None
    # data type: IMetadataModelLocationMetadataModel
    oneof_schema_5_validator: Optional[IMetadataModelLocationMetadataModel] = None
    # data type: IMetadataModelOriginalFilenameMetadataModel
    oneof_schema_6_validator: Optional[IMetadataModelOriginalFilenameMetadataModel] = None
    # data type: IMetadataModelSourceUrlMetadataModel
    oneof_schema_7_validator: Optional[IMetadataModelSourceUrlMetadataModel] = None
    # data type: IMetadataModelStreamsMetadataModel
    oneof_schema_8_validator: Optional[IMetadataModelStreamsMetadataModel] = None
    # data type: IMetadataModelTextMetadataModel
    oneof_schema_9_validator: Optional[IMetadataModelTextMetadataModel] = None
    # data type: IMetadataModelVideoDurationMetadataModel
    oneof_schema_10_validator: Optional[IMetadataModelVideoDurationMetadataModel] = None
    actual_instance: Optional[Union[IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel]] = None
    one_of_schemas: Set[str] = { "IMetadataModelClassificationMetadataModel", "IMetadataModelCountMetadataModel", "IMetadataModelFileTypeMetadataModel", "IMetadataModelImageDimensionMetadataModel", "IMetadataModelLocationMetadataModel", "IMetadataModelOriginalFilenameMetadataModel", "IMetadataModelSourceUrlMetadataModel", "IMetadataModelStreamsMetadataModel", "IMetadataModelTextMetadataModel", "IMetadataModelVideoDurationMetadataModel" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = IMetadataModel.model_construct()
        error_messages = []
        match = 0
        # validate data type: IMetadataModelClassificationMetadataModel
        if not isinstance(v, IMetadataModelClassificationMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelClassificationMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelCountMetadataModel
        if not isinstance(v, IMetadataModelCountMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelCountMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelFileTypeMetadataModel
        if not isinstance(v, IMetadataModelFileTypeMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelFileTypeMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelImageDimensionMetadataModel
        if not isinstance(v, IMetadataModelImageDimensionMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelImageDimensionMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelLocationMetadataModel
        if not isinstance(v, IMetadataModelLocationMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelLocationMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelOriginalFilenameMetadataModel
        if not isinstance(v, IMetadataModelOriginalFilenameMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelOriginalFilenameMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelSourceUrlMetadataModel
        if not isinstance(v, IMetadataModelSourceUrlMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelSourceUrlMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelStreamsMetadataModel
        if not isinstance(v, IMetadataModelStreamsMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelStreamsMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelTextMetadataModel
        if not isinstance(v, IMetadataModelTextMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelTextMetadataModel`")
        else:
            match += 1
        # validate data type: IMetadataModelVideoDurationMetadataModel
        if not isinstance(v, IMetadataModelVideoDurationMetadataModel):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IMetadataModelVideoDurationMetadataModel`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in IMetadataModel with oneOf schemas: IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in IMetadataModel with oneOf schemas: IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into IMetadataModelClassificationMetadataModel
        try:
            instance.actual_instance = IMetadataModelClassificationMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelCountMetadataModel
        try:
            instance.actual_instance = IMetadataModelCountMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelFileTypeMetadataModel
        try:
            instance.actual_instance = IMetadataModelFileTypeMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelImageDimensionMetadataModel
        try:
            instance.actual_instance = IMetadataModelImageDimensionMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelLocationMetadataModel
        try:
            instance.actual_instance = IMetadataModelLocationMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelOriginalFilenameMetadataModel
        try:
            instance.actual_instance = IMetadataModelOriginalFilenameMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelSourceUrlMetadataModel
        try:
            instance.actual_instance = IMetadataModelSourceUrlMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelStreamsMetadataModel
        try:
            instance.actual_instance = IMetadataModelStreamsMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelTextMetadataModel
        try:
            instance.actual_instance = IMetadataModelTextMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IMetadataModelVideoDurationMetadataModel
        try:
            instance.actual_instance = IMetadataModelVideoDurationMetadataModel.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into IMetadataModel with oneOf schemas: IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into IMetadataModel with oneOf schemas: IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], IMetadataModelClassificationMetadataModel, IMetadataModelCountMetadataModel, IMetadataModelFileTypeMetadataModel, IMetadataModelImageDimensionMetadataModel, IMetadataModelLocationMetadataModel, IMetadataModelOriginalFilenameMetadataModel, IMetadataModelSourceUrlMetadataModel, IMetadataModelStreamsMetadataModel, IMetadataModelTextMetadataModel, IMetadataModelVideoDurationMetadataModel]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class CreateDatapointsFromS3BucketModel(BaseModel):
    """
    The model for creating datapoints from files in a S3 bucket.
    """ # noqa: E501
    bucket_name: StrictStr = Field(description="The name of the S3 bucket to upload the files from.", alias="bucketName")
    region: Optional[StrictStr] = Field(default=None, description="The region of the S3 bucket.")
    source_prefix: StrictStr = Field(description="The prefix of the files to upload.", alias="sourcePrefix")
    access_key: Optional[StrictStr] = Field(default=None, description="The access key to use for the S3 bucket.", alias="accessKey")
    secret_key: Optional[StrictStr] = Field(default=None, description="The secret key to use for the S3 bucket.", alias="secretKey")
    use_custom_aws_credentials: Optional[StrictBool] = Field(default=None, description="Whether to use custom AWS credentials.", alias="useCustomAwsCredentials")
    clear_dataset: Optional[StrictBool] = Field(default=None, description="Whether to clear the dataset before uploading the files.", alias="clearDataset")
    __properties: ClassVar[List[str]] = ["bucketName", "region", "sourcePrefix", "accessKey", "secretKey", "useCustomAwsCredentials", "clearDataset"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateDatapointsFromS3BucketModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if region (nullable) is None
        # and model_fields_set contains the field
        if self.region is None and "region" in self.model_fields_set:
            _dict['region'] = None

        # set to None if access_key (nullable) is None
        # and model_fields_set contains the field
        if self.access_key is None and "access_key" in self.model_fields_set:
            _dict['accessKey'] = None

        # set to None if secret_key (nullable) is None
        # and model_fields_set contains the field
        if self.secret_key is None and "secret_key" in self.model_fields_set:
            _dict['secretKey'] = None

        # set to None if use_custom_aws_credentials (nullable) is None
        # and model_fields_set contains the field
        if self.use_custom_aws_credentials is None and "use_custom_aws_credentials" in self.model_fields_set:
            _dict['useCustomAwsCredentials'] = None

        # set to None if clear_dataset (nullable) is None
        # and model_fields_set contains the field
        if self.clear_dataset is None and "clear_dataset" in self.model_fields_set:
            _dict['clearDataset'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateDatapointsFromS3BucketModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "bucketName": obj.get("bucketName"),
            "region": obj.get("region"),
            "sourcePrefix": obj.get("sourcePrefix"),
            "accessKey": obj.get("accessKey"),
            "secretKey": obj.get("secretKey"),
            "useCustomAwsCredentials": obj.get("useCustomAwsCredentials"),
            "clearDataset": obj.get("clearDataset")
        })
        return _obj





from __future__ import annotations

from typing import TYPE_CHECKING, Literal, Optional, Sequence, get_args
import secrets

from rapidata.rapidata_client.datapoints._datapoint import Datapoint

if TYPE_CHECKING:
    from rapidata.api_client.models.create_order_model import CreateOrderModel
from rapidata.rapidata_client.exceptions.failed_upload_exception import (
    FailedUploadException,
)
from rapidata.rapidata_client.filter import RapidataFilter, UserScoreFilter
from rapidata.rapidata_client.config import (
    logger,
    managed_print,
    rapidata_config,
    tracer,
)
from rapidata.rapidata_client.validation.validation_set_manager import (
    ValidationSetManager,
)
from rapidata.rapidata_client.validation.rapidata_validation_set import (
    RapidataValidationSet,
)
from rapidata.rapidata_client.dataset._rapidata_dataset import RapidataDataset
from rapidata.rapidata_client.order.rapidata_order import RapidataOrder
from rapidata.rapidata_client.referee import Referee
from rapidata.rapidata_client.referee._naive_referee import NaiveReferee
from rapidata.rapidata_client.selection._base_selection import RapidataSelection
from rapidata.rapidata_client.settings import RapidataSetting
from rapidata.rapidata_client.workflow import Workflow
from rapidata.rapidata_client.selection import (
    ConditionalValidationSelection,
    LabelingSelection,
    CappedSelection,
)
from rapidata.service.openapi_service import OpenAPIService
from rapidata.rapidata_client.api.rapidata_api_client import (
    suppress_rapidata_error_logging,
)

StickyStateLiteral = Literal["Temporary", "Permanent", "Passive"]


class RapidataOrderBuilder:
    """Builder object for creating Rapidata orders.

    Use the fluent interface to set the desired configuration. Add a workflow to the order using `.workflow()` and finally call `.create()` to create the order.

    Args:
        openapi_service (OpenAPIService): The OpenAPIService instance.
        name (str): The name of the order.
    """

    def __init__(
        self,
        name: str,
        openapi_service: OpenAPIService,
    ):
        self._name = name
        self._openapi_service = openapi_service
        self._dataset: Optional[RapidataDataset] = None
        self._workflow: Workflow | None = None
        self._referee: Referee | None = None
        self._validation_set: RapidataValidationSet | None = None
        self._settings: Sequence[RapidataSetting] | None = None
        self._user_filters: list[RapidataFilter] = []
        self._selections: Sequence[RapidataSelection] = []
        self._priority: int | None = None
        self._datapoints: list[Datapoint] = []
        self._sticky_state_value: StickyStateLiteral | None = None
        self._validation_set_manager: ValidationSetManager = ValidationSetManager(
            self._openapi_service
        )

    def _to_model(self) -> CreateOrderModel:
        """
        Convert the builder configuration to a CreateOrderModel.

        Raises:
            ValueError: If no workflow is provided.

        Returns:
            CreateOrderModel: The model representing the order configuration.
        """
        if self._workflow is None:
            raise ValueError("You must provide a workflow to create an order.")

        if self._referee is None:
            managed_print("No referee provided, using default NaiveReferee.")
            self._referee = NaiveReferee()

        sticky_state = self._sticky_state_value

        validation_set_id = (
            self._validation_set.id
            if (self._validation_set and not self._selections)
            else None
        )
        from rapidata.api_client.models.create_order_model import CreateOrderModel
        from rapidata.api_client.models.sticky_state import StickyState

        return CreateOrderModel(
            orderName=self._name,
            workflow=self._workflow._to_model(),
            userFilters=[user_filter._to_model() for user_filter in self._user_filters],
            referee=self._referee._to_model(),
            validationSetId=validation_set_id,
            featureFlags=(
                [setting._to_feature_flag_model() for setting in self._settings]
                if self._settings is not None
                else None
            ),
            selections=(
                [selection._to_model() for selection in self._selections]
                if self._selections
                else None
            ),
            priority=self._priority,
            stickyState=(StickyState(sticky_state) if sticky_state else None),
        )

    def _generate_id(self, length=9):
        """Optimal balance: fast comparisons + low collision risk"""
        alphabet = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
        return "".join(secrets.choice(alphabet) for _ in range(length))

    def _create(self) -> RapidataOrder:
        """
        Create the Rapidata order by making the necessary API calls based on the builder's configuration.

        Raises:
            ValueError: If both media paths and texts are provided, or if neither is provided.
            AssertionError: If the workflow is a CompareWorkflow and media paths are not in pairs.

        Returns:
            RapidataOrder: The created RapidataOrder instance.
        """
        order_model = self._to_model()
        logger.debug("Creating order with model: %s", order_model)

        result = self._openapi_service.order_api.order_post(
            create_order_model=order_model
        )

        order = RapidataOrder(
            order_id=str(result.order_id),
            openapi_service=self._openapi_service,
            name=self._name,
        )
        logger.debug("Order created: %s", order)

        if not result.dataset_id:
            logger.error("No Dataset was created for order %s", order.id)
            return order

        self._dataset = RapidataDataset(result.dataset_id, self._openapi_service)

        logger.debug("Dataset created with ID: %s", self._dataset.id)
        logger.debug("Adding datapoints to the order.")
        with tracer.start_as_current_span("add_datapoints"):
            _, failed_uploads = self._dataset.add_datapoints(self._datapoints)

            if failed_uploads:
                raise FailedUploadException(self._dataset, failed_uploads, order=order)

        logger.debug("Datapoints added to the order.")
        logger.debug("Setting order to preview")
        try:
            self._openapi_service.order_api.order_order_id_preview_post(order.id)
        except Exception:
            raise FailedUploadException(self._dataset, failed_uploads, order=order)
        return order

    def _set_workflow(self, workflow: Workflow) -> RapidataOrderBuilder:
        """
        Set the workflow for the order.

        Args:
            workflow (Workflow): The workflow to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if not isinstance(workflow, Workflow):
            raise TypeError("Workflow must be of type Workflow.")

        self._workflow = workflow
        return self

    def _set_referee(self, referee: Referee) -> "RapidataOrderBuilder":
        """
        Set the referee for the order.

        Args:
            referee (Referee): The referee to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if not isinstance(referee, Referee):
            raise TypeError("Referee must be of type Referee.")

        self._referee = referee
        return self

    def _set_datapoints(
        self,
        datapoints: list[Datapoint],
    ) -> RapidataOrderBuilder:
        """
        Set the datapoints for the order.

        Args:
            datapoints: (Sequence[Datapoint]): The datapoints to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if not isinstance(datapoints, list):
            raise TypeError(
                "Datapoints must be provided as a list of Datapoint objects."
            )

        self._datapoints = datapoints
        return self

    def _set_settings(
        self, settings: Sequence[RapidataSetting]
    ) -> RapidataOrderBuilder:
        """
        Set the settings for the order.

        Args:
            settings (Settings): The settings to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """

        if not isinstance(settings, list):
            raise TypeError("Settings must be provided as a list of Setting objects.")

        for s in settings:
            if not isinstance(s, RapidataSetting):
                raise TypeError("The settings list must only contain Setting objects.")

        self._settings = settings
        return self

    def _set_filters(self, filters: Sequence[RapidataFilter]) -> RapidataOrderBuilder:
        """
        Set the filters for the order, e.g., country, language, userscore, etc.

        Args:
            filters (Sequence[Filters]): The user filters to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if not isinstance(filters, list):
            raise TypeError("Filters must be provided as a list of Filter objects.")

        for f in filters:
            if not isinstance(f, RapidataFilter):
                raise TypeError("Filters must be of type Filter.")

        if len(self._user_filters) > 0:
            managed_print("Overwriting existing user filters.")

        self._user_filters = filters
        return self

    def _set_validation_set_id(
        self, validation_set_id: str | None = None
    ) -> RapidataOrderBuilder:
        """
        Set the validation set ID for the order.

        Args:
            validation_set_id (str): The validation set ID to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if validation_set_id is not None and not isinstance(validation_set_id, str):
            raise TypeError("Validation set ID must be of type str.")
        if validation_set_id is not None:
            self._validation_set = (
                self._validation_set_manager.get_validation_set_by_id(validation_set_id)
            )
            return self
        self._validation_set = validation_set_id
        return self

    def _set_selections(
        self, selections: Sequence[RapidataSelection]
    ) -> RapidataOrderBuilder:
        """
        Set the selections for the order.

        Args:
            selections (Sequence[Selection]): The selections to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if not isinstance(selections, list):
            raise TypeError(
                "Selections must be provided as a list of Selection objects."
            )

        for selection in selections:
            if not isinstance(selection, RapidataSelection):
                raise TypeError("Selections must be of type Selection.")

        self._selections = selections  # type: ignore
        return self

    def _set_priority(self, priority: int | None = None) -> RapidataOrderBuilder:
        """
        Set the priority for the order.

        Args:
            priority (int): The priority to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        if priority is not None and not isinstance(priority, int):
            raise TypeError("Priority must be of type int.")

        self._priority = priority
        return self

    def _set_sticky_state(
        self, sticky_state: StickyStateLiteral | None = None
    ) -> RapidataOrderBuilder:
        """
        Set the sticky state for the order.
        """
        sticky_state_valid_values = get_args(StickyStateLiteral)

        if sticky_state is not None and sticky_state not in sticky_state_valid_values:
            raise ValueError(
                f"Sticky state must be one of {sticky_state_valid_values} or None"
            )

        self._sticky_state_value = sticky_state
        return self



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.i_selection_demographic_selection import ISelectionDemographicSelection
from rapidata.api_client.models.i_selection_effort_capped_selection import ISelectionEffortCappedSelection
from rapidata.api_client.models.i_selection_labeling_selection import ISelectionLabelingSelection
from rapidata.api_client.models.i_selection_static_selection import ISelectionStaticSelection
from rapidata.api_client.models.i_selection_validation_selection import ISelectionValidationSelection
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

ISELECTION_ONE_OF_SCHEMAS = ["ISelectionAbTestSelection", "ISelectionCappedSelection", "ISelectionConditionalValidationSelection", "ISelectionDemographicSelection", "ISelectionEffortCappedSelection", "ISelectionLabelingSelection", "ISelectionShufflingSelection", "ISelectionStaticSelection", "ISelectionValidationSelection"]

class ISelection(BaseModel):
    """
    ISelection
    """
    # data type: ISelectionAbTestSelection
    oneof_schema_1_validator: Optional[ISelectionAbTestSelection] = None
    # data type: ISelectionCappedSelection
    oneof_schema_2_validator: Optional[ISelectionCappedSelection] = None
    # data type: ISelectionConditionalValidationSelection
    oneof_schema_3_validator: Optional[ISelectionConditionalValidationSelection] = None
    # data type: ISelectionDemographicSelection
    oneof_schema_4_validator: Optional[ISelectionDemographicSelection] = None
    # data type: ISelectionEffortCappedSelection
    oneof_schema_5_validator: Optional[ISelectionEffortCappedSelection] = None
    # data type: ISelectionLabelingSelection
    oneof_schema_6_validator: Optional[ISelectionLabelingSelection] = None
    # data type: ISelectionShufflingSelection
    oneof_schema_7_validator: Optional[ISelectionShufflingSelection] = None
    # data type: ISelectionStaticSelection
    oneof_schema_8_validator: Optional[ISelectionStaticSelection] = None
    # data type: ISelectionValidationSelection
    oneof_schema_9_validator: Optional[ISelectionValidationSelection] = None
    actual_instance: Optional[Union[ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection]] = None
    one_of_schemas: Set[str] = { "ISelectionAbTestSelection", "ISelectionCappedSelection", "ISelectionConditionalValidationSelection", "ISelectionDemographicSelection", "ISelectionEffortCappedSelection", "ISelectionLabelingSelection", "ISelectionShufflingSelection", "ISelectionStaticSelection", "ISelectionValidationSelection" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = ISelection.model_construct()
        error_messages = []
        match = 0
        # validate data type: ISelectionAbTestSelection
        if not isinstance(v, ISelectionAbTestSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionAbTestSelection`")
        else:
            match += 1
        # validate data type: ISelectionCappedSelection
        if not isinstance(v, ISelectionCappedSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionCappedSelection`")
        else:
            match += 1
        # validate data type: ISelectionConditionalValidationSelection
        if not isinstance(v, ISelectionConditionalValidationSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionConditionalValidationSelection`")
        else:
            match += 1
        # validate data type: ISelectionDemographicSelection
        if not isinstance(v, ISelectionDemographicSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionDemographicSelection`")
        else:
            match += 1
        # validate data type: ISelectionEffortCappedSelection
        if not isinstance(v, ISelectionEffortCappedSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionEffortCappedSelection`")
        else:
            match += 1
        # validate data type: ISelectionLabelingSelection
        if not isinstance(v, ISelectionLabelingSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionLabelingSelection`")
        else:
            match += 1
        # validate data type: ISelectionShufflingSelection
        if not isinstance(v, ISelectionShufflingSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionShufflingSelection`")
        else:
            match += 1
        # validate data type: ISelectionStaticSelection
        if not isinstance(v, ISelectionStaticSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionStaticSelection`")
        else:
            match += 1
        # validate data type: ISelectionValidationSelection
        if not isinstance(v, ISelectionValidationSelection):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ISelectionValidationSelection`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in ISelection with oneOf schemas: ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in ISelection with oneOf schemas: ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into ISelectionAbTestSelection
        try:
            instance.actual_instance = ISelectionAbTestSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionCappedSelection
        try:
            instance.actual_instance = ISelectionCappedSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionConditionalValidationSelection
        try:
            instance.actual_instance = ISelectionConditionalValidationSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionDemographicSelection
        try:
            instance.actual_instance = ISelectionDemographicSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionEffortCappedSelection
        try:
            instance.actual_instance = ISelectionEffortCappedSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionLabelingSelection
        try:
            instance.actual_instance = ISelectionLabelingSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionShufflingSelection
        try:
            instance.actual_instance = ISelectionShufflingSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionStaticSelection
        try:
            instance.actual_instance = ISelectionStaticSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ISelectionValidationSelection
        try:
            instance.actual_instance = ISelectionValidationSelection.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into ISelection with oneOf schemas: ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into ISelection with oneOf schemas: ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], ISelectionAbTestSelection, ISelectionCappedSelection, ISelectionConditionalValidationSelection, ISelectionDemographicSelection, ISelectionEffortCappedSelection, ISelectionLabelingSelection, ISelectionShufflingSelection, ISelectionStaticSelection, ISelectionValidationSelection]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

from rapidata.api_client.models.i_selection_ab_test_selection import ISelectionAbTestSelection
from rapidata.api_client.models.i_selection_capped_selection import ISelectionCappedSelection
from rapidata.api_client.models.i_selection_conditional_validation_selection import ISelectionConditionalValidationSelection
from rapidata.api_client.models.i_selection_shuffling_selection import ISelectionShufflingSelection
# TODO: Rewrite to not use raise_errors
ISelection.model_rebuild(raise_errors=False)

